@@ -1,16 +1,124 @@
 import os
 import io
+import re
 import logging
 import datetime
 import codecs
 import csv
 import json
+import requests
 
-from . import urlquote, datapath, DEFAULT_HEADERS, DEFAULT_CHUNK_SIZE, DEFAULT_SESSION_CONFIG, Megabyte, Kilobyte, \
-    get_transfer_summary, IS_PY2
-from .deriva_binding import DerivaBinding
+from . import urlquote, urlsplit, urlunsplit, datapath, DEFAULT_HEADERS, DEFAULT_CHUNK_SIZE, DEFAULT_SESSION_CONFIG, \
+    Megabyte, Kilobyte, get_transfer_summary, IS_PY2
+from .deriva_binding import DerivaBinding, DerivaPathError
 from . import ermrest_model
+from .ermrest_model import nochange
 
+class DerivaServer (DerivaBinding):
+    """Persistent handle for a Deriva server."""
+
+    def __init__(self, scheme, server, credentials=None, caching=True, session_config=None):
+        """Create a Deriva server binding.
+
+           Arguments:
+             scheme: 'http' or 'https'
+             server: server FQDN string
+             credentials: credential secrets, e.g. cookie
+             caching: whether to retain a GET response cache
+        """
+        super(DerivaServer, self).__init__(scheme, server, credentials, caching, session_config)
+        self.scheme = scheme
+        self.server = server
+        self.credentials = credentials
+        self.caching = caching
+        self.session_config = session_config
+
+    def connect_ermrest(self, catalog_id, snaptime=None):
+        """Connect to an ERMrest catalog and return the catalog binding.
+
+        :param catalog_id: The id (or alias) of the existing catalog
+        :param snaptime: The id for a desired catalog snapshot (default None)
+
+        The catalog_id is normally a bare id (str), and the optional
+        snaptime is a bare snapshot id (str). If the snaptime is None,
+        the catalog_id may be a concatenated <id>@<snaptime> string,
+        and it will be split to determine the snaptime.
+
+        If no snaptime is passed separately or compounded with
+        catalog_id, an ErmrestCatalog binding will be
+        returned. Conversely, if a snaptime is determined, an
+        ErmrestSnapshot (immutable) binding will be returned.
+
+        """
+        return ErmrestCatalog.connect(self, catalog_id, snaptime)
+
+    def create_ermrest_catalog(self, id=None, owner=None):
+        """Create an ERMrest catalog.
+
+        :param id: The (str) id desired by the client (default None)
+        :param owner: The initial (list of str) ACL desired by the client (default None)
+
+        The new catalog id will be returned in the response, and used
+        in future catalog access. The use of the id parameter
+        may yield errors if the supplied value is not available for
+        use by the client. The value None will result in a
+        server-assigned catalog id.
+
+        The initial "owner" ACL on the new catalog will be the
+        client-supplied owner if provided. The use of owner parameter
+        may yield errors if the supplied ACL does not match the
+        client, i.e. the client cannot lock themselves out of the
+        catalog. The value None will result in a server-assigned ACL
+        with the requesting client's identity.
+
+        Certain failure modes (or message loss) may leave the
+        id reserved in the system. In this case, the effective
+        owner ACL influences which client(s) are allowed to retry
+        creation with the same id.
+
+        """
+        return ErmrestCatalog.create(self, id, owner)
+
+    def connect_ermrest_alias(self, id):
+        """Connect to an ERMrest alias and return the alias binding.
+
+        :param id: The id of the existing alias
+
+        """
+        return ErmrestAlias.connect(self, id)
+
+    def create_ermrest_alias(self, id=None, owner=None, alias_target=None):
+        """Create an ERMrest catalog alias.
+
+        :param id: The (str) id desired by the client (default None)
+        :param owner: The initial (list of str) ACL desired by the client (default None)
+        :param alias_target: The initial target catalog id binding desired by the client (default None)
+
+        The new alias id will be returned in the response, and used
+        in future alias access. The use of the id parameter
+        may yield errors if the supplied value is not available for
+        use by the client. The value None will result in a
+        server-assigned alias id.
+
+        The initial "owner" ACL on the new alias will be the
+        client-supplied owner. The use of owner parameter may yield
+        errors if the supplied ACL does not match the client, i.e. the
+        client cannot lock themselves out of the alias. The value
+        None will result in a server-assigned ACL with the requesting
+        client's identity.
+
+        The alias is bound to the client-supplied alias_target, if
+        supplied. The use of alias_target may yield errors if the
+        supplied value is not a valid target catalog id. The value
+        None will reserve the alias in an unbound state.
+
+        Certain failure modes (or message loss) may leave the id
+        reserved in the system. In this case, the effective owner_acl
+        influences which client(s) are allowed to retry creation with
+        the same id.
+
+        """
+        return ErmrestAlias.create(self, id, owner, alias_target)
 
 class ErmrestCatalogMutationError(Exception):
     pass
@@ -32,6 +140,118 @@ class ErmrestCatalog(DerivaBinding):
     """
     table_schemas = dict()
 
+    @property
+    def deriva_server(self):
+        """Return DerivaServer binding for the same server this catalog belongs to."""
+        return DerivaServer(
+            self._scheme,
+            self._server,
+            self._credentials,
+            self._caching,
+            self._session_config,
+        )
+
+    @classmethod
+    def connect(cls, deriva_server, catalog_id, snaptime=None):
+        """Connect to an ERMrest catalog and return the catalog binding.
+
+        :param deriva_server: The DerivaServer binding which hosts ermrest
+        :param catalog_id: The id (or alias) of the existing catalog
+        :param snaptime: The id for a desired catalog snapshot (default None)
+
+        The catalog_id is normally a bare id (str), and the optional
+        snaptime is a bare snapshot id (str). If the snaptime is None,
+        the catalog_id may be a concatenated <id>@<snaptime> string,
+        and it will be split to determine the snaptime.
+
+        If no snaptime is passed separately or compounded with
+        catalog_id, an ErmrestCatalog binding will be
+        returned. Conversely, if a snaptime is determined, an
+        ErmrestSnapshot (immutable) binding will be returned.
+
+        """
+        if not snaptime:
+            splits = str(catalog_id).split('@')
+            if len(splits) > 2:
+                raise Exception('Malformed catalog identifier: multiple "@" characters found.')
+            catalog_id = splits[0]
+            snaptime = splits[1] if len(splits) == 2 else None
+
+        if snaptime:
+            return ErmrestSnapshot(
+                deriva_server.scheme,
+                deriva_server.server,
+                catalog_id,
+                snaptime,
+                deriva_server.credentials,
+                deriva_server.caching,
+                deriva_server.session_config
+            )
+
+        return cls(
+            deriva_server.scheme,
+            deriva_server.server,
+            catalog_id,
+            deriva_server.credentials,
+            deriva_server.caching,
+            deriva_server.session_config
+        )
+
+    @classmethod
+    def _digest_catalog_args(cls, id, owner):
+        rep = dict()
+
+        if isinstance(id, str):
+            rep['id'] = id
+        elif isinstance(id, (type(nochange), type(None))):
+            pass
+        else:
+            raise TypeError('id must be of type str or None or nochange, not %s' % type(id))
+
+        if isinstance(owner, list):
+            for e in owner:
+                if not isinstance(e, str):
+                    raise TypeError('owner members must be of type str, not %s' % type(e))
+            rep['owner'] = owner
+        elif isinstance(owner, (type(nochange), type(None))):
+            pass
+        else:
+            raise TypeError('owner must be of type list or None or nochange, not %s' % type(owner))
+
+        return rep
+
+    @classmethod
+    def create(cls, deriva_server, id=None, owner=None):
+        """Create an ERMrest catalog and return the ERMrest catalog binding.
+
+        :param deriva_server: The DerivaServer binding which hosts ermrest.
+        :param id: The (str) id desired by the client (default None)
+        :param owner: The initial (list of str) ACL desired by the client (default None)
+
+        The new catalog id will be returned in the response, and used
+        in future catalog access. The use of the id parameter
+        may yield errors if the supplied value is not available for
+        use by the client. The value None will result in a
+        server-assigned catalog id.
+
+        The initial "owner" ACL on the new catalog will be the
+        client-supplied owner ACL. The use of owner parameter
+        may yield errors if the supplied ACL does not match the
+        client, i.e. the client cannot lock themselves out of the
+        catalog. The value None will result in a server-assigned ACL
+        with the requesting client's identity.
+
+        Certain failure modes (or message loss) may leave the id
+        reserved in the system. In this case, the effective owner ACL
+        influences which client(s) are allowed to retry creation with
+        the same id.
+
+        """
+        path = '/ermrest/catalog'
+        r = deriva_server.post(path, json=cls._digest_catalog_args(id, owner))
+        r.raise_for_status()
+        return cls.connect(deriva_server, r.json()['id'])
+
     def __init__(self, scheme, server, catalog_id, credentials=None, caching=True, session_config=None):
         """Create ERMrest catalog binding.
 
@@ -49,7 +269,7 @@ class ErmrestCatalog(DerivaBinding):
              self.dcctx['cid'] = 'my application name'
 
            You MAY also supply custom per-request context by passing a
-           headers dict to web request methods, e.g. 
+           headers dict to web request methods, e.g.
 
              self.get(..., headers={'deriva-client-context': {'action': 'myapp/function1'}})
 
@@ -58,9 +278,11 @@ class ErmrestCatalog(DerivaBinding):
            complete context for the request.
         """
         super(ErmrestCatalog, self).__init__(scheme, server, credentials, caching, session_config)
+        if isinstance(catalog_id, int):
+            catalog_id = str(catalog_id)
         self._server_uri = "%s/ermrest/catalog/%s" % (
             self._server_uri,
-            catalog_id
+            urlquote(catalog_id),
         )
         self._scheme, self._server, self._catalog_id, self._credentials, self._caching, self._session_config = \
             scheme, server, catalog_id, credentials, caching, session_config
@@ -69,6 +291,13 @@ class ErmrestCatalog(DerivaBinding):
     def catalog_id(self):
         return self._catalog_id
 
+    @property
+    def alias_target(self):
+        r = self.get('/')
+        r.raise_for_status()
+        rep = r.json()
+        return rep.get('alias_target')
+
     def exists(self):
         """Simple boolean test for catalog existence.
 
@@ -77,7 +306,7 @@ class ErmrestCatalog(DerivaBinding):
         try:
             self.get('/')
             return True
-        except HTTPError as e:
+        except requests.HTTPError as e:
             if e.response.status_code == 404:
                 return False
             else:
@@ -164,10 +393,11 @@ class ErmrestCatalog(DerivaBinding):
                   callback=None,
                   delete_if_empty=False,
                   paged=False,
-                  page_size=DEFAULT_PAGE_SIZE):
+                  page_size=DEFAULT_PAGE_SIZE,
+                  page_sort_columns=frozenset(["RID"])):
         """
            Retrieve catalog data streamed to destination file.
-           Caller is responsible to clean up file even on error, when the file may or may not be exist.
+           Caller is responsible to clean up file even on error, when the file may or may not exist.
            If "delete_if_empty" is True, the file will be inspected for "empty" content. In the case of
            json/json-stream content, the presence of a single empty JSON object will be tested for. In the case of
            CSV content, the file will be parsed with CSV reader to determine that only a single header line and no row
@@ -179,8 +409,8 @@ class ErmrestCatalog(DerivaBinding):
         # exception in the case that the caller might be trying to perform an opportunistic paged request without
         # knowing a priori if paged support for the given query is available.
         page_size = page_size if page_size > 0 else DEFAULT_PAGE_SIZE
-        if not path.startswith("/entity") and paged:
-            logging.debug("Paged data retrieval only supported for entity API queries.")
+        if not (path.startswith("/entity") or path.startswith("/attribute")) and paged:
+            logging.warning("Paged data retrieval only supported for entity or attribute API queries.")
             paged = False
 
         # Only "application/x-json-stream" or "text/csv" supported with paged mode at this time, otherwise fallback.
@@ -196,6 +426,7 @@ class ErmrestCatalog(DerivaBinding):
         try:
             total = 0
             start = datetime.datetime.now()
+
             if not paged:
                 with self._session.get(self._server_uri + path, headers=headers, stream=True) as r:
                     self._response_raise_for_status(r)
@@ -210,29 +441,42 @@ class ErmrestCatalog(DerivaBinding):
                                 destfile.close()
                                 return
                 destfile.flush()
-                os.fsync(destfile.fileno())
             else:
                 first_page = True
                 first_line = None
                 last_record = None
+                usr = urlsplit(self._server_uri + path)
+                path = str(usr.path.split('@sort')[0])
                 while True:
-                    qualifier = "@sort(RID)%s?limit=%d" % \
-                               (("@after(%s)" % urlquote(last_record)) if last_record is not None else "", page_size)
+                    sort = "@sort(%s)%s" % (",".join(page_sort_columns or ["RID"]),
+                                            ("@after(%s)" % ",".join(last_record)) if last_record is not None else "")
+                    limit = "limit=%s" % int(page_size) if page_size > 0 else "none"
+                    query = re.sub(r"([^.]*)(limit=.*?)($|[&;])([^.]*)$", r"\1%s\3\4" % limit, usr.query, flags=re.I)
+                    url = urlunsplit((usr.scheme, usr.netloc, path + sort, query if query else limit, usr.fragment))
+
                     # 1. Try to get a page worth of data, back-off page size if query run time errors are encountered
-                    with self._session.get(self._server_uri + path + qualifier, headers=headers) as r:
+                    with self._session.get(url, headers=headers) as r:
                         if r.status_code == 400 and "Query run time limit exceeded" in r.text:
+                            if page_size == 1:
+                                self._response_raise_for_status(r)
                             r.close()
                             page_size //= 2
                             page_size = 1 if page_size < 1 else page_size
                             logging.warning("Query runtime exceeded while attempting to transfer rows from %s to file "
                                             "[%s]. The page size is being reduced to %s and the query will be retried."
-                                            % (self._server_uri + path, destfilename, page_size))
+                                            % (url, destfilename, page_size))
+                            if callback:
+                                if not callback(progress="Retrying query: %s" % url):
+                                    destfile.close()
+                                    return
                             continue
+                        else:
+                            self._response_raise_for_status(r)
 
-                        # 2. Write the page to disk and determine the last RID processed in order to get the next page
-                        last_line = None
+                        # 2. Write the page to disk and check the last record processed in order to get the next page
+                        last_line = {}
                         content_type = r.headers.get("Content-Type")
-                        logging.debug("Transferring file %s to %s" % (self._server_uri + path, destfilename))
+                        logging.debug("Transferring data from [%s] to %s" % (url, destfilename))
                         # CSV processing iterates over lines in the response, skipping the header line(s) in all but
                         # the first page, and captures the last line of each page to determine the last record processed
                         if content_type == "text/csv":
@@ -273,12 +517,11 @@ class ErmrestCatalog(DerivaBinding):
                                     break
                             last_line = json.loads(b.readline().decode('utf-8'))
 
-                        # 3. Save the last record RID and flush the destination file buffers to disk.
+                        # 3. Save the last record key and flush the destination file buffers to disk.
                         if not last_line:
                             break
                         destfile.flush()
-                        os.fsync(destfile.fileno())
-                        last_record = last_line['RID']
+                        last_record = [urlquote(str(last_line.get(key))) for key in page_sort_columns]
                         if callback:
                             if not callback(progress="Downloading: %.2f MB transferred" %
                                                      (float(total) / float(Megabyte))):
@@ -673,3 +916,157 @@ class ErmrestSnapshot(ErmrestCatalog):
         When called by the super-class, this method raises an exception.
         """
         raise ErmrestCatalogMutationError('Catalog snapshot is immutable')
+
+class ErmrestAlias(DerivaBinding):
+    """Persistent handle for an ERMrest alias.
+
+       Provides basic REST client for HTTP methods on arbitrary
+       paths. Caller has to understand ERMrest APIs and compose
+       appropriate paths, headers, and/or content.
+
+       Additional utility methods provided for accessing alias metadata.
+    """
+    @classmethod
+    def connect(cls, deriva_server, alias_id):
+        """Connect to an ERMrest alias and return the alias binding.
+
+        :param deriva_server: The DerivaServer binding which hosts ermrest
+        :param alias_id: The id of the existing alias
+
+        The alias_id is a bare id (str).
+
+        """
+        return cls(
+            deriva_server.scheme,
+            deriva_server.server,
+            alias_id,
+            deriva_server.credentials,
+            deriva_server.caching,
+            deriva_server.session_config
+        )
+
+    @classmethod
+    def _digest_alias_args(cls, id, owner, alias_target):
+        rep = ErmrestCatalog._digest_catalog_args(id, owner)
+
+        if isinstance(alias_target, (str, type(None))):
+            rep['alias_target'] = alias_target
+        elif isinstance(alias_target, type(nochange)):
+            pass
+        else:
+            raise TypeError('alias_target must be of type str or None or nochange, not %s' % type(alias_target))
+
+        return rep
+
+    @classmethod
+    def create(cls, deriva_server, id=None, owner=None, alias_target=None):
+        """Create an ERMrest catalog alias.
+
+        :param deriva_server: The DerivaServer binding which hosts ermrest
+        :param id: The (str) id desired by the client (default None)
+        :param owner: The initial (list of str) ACL desired by the client (default None)
+        :param alias_target: The initial target catalog id desired by the client (default None)
+
+        The new alias id will be returned in the response, and used
+        in future alias access. The use of the id parameter
+        may yield errors if the supplied value is not available for
+        use by the client. The value None will result in a
+        server-assigned alias id.
+
+        The initial "owner" ACL on the new alias will be the
+        client-supplied owner parameter. The use of owner may yield
+        errors if the supplied ACL does not match the client, i.e. the
+        client cannot lock themselves out of the alias. The value None
+        will result in a server-assigned ACL with the requesting
+        client's identity.
+
+        The alias is bound to the client-supplied alias_target, if
+        supplied. The use of alias_target may yield errors if the
+        supplied value is not a valid target catalog id. The value
+        None will reserve the alias in an unbound state.
+
+        Certain failure modes (or message loss) may leave the id
+        reserved in the system. In this case, the effective owner ACL
+        influences which client(s) are allowed to retry creation with
+        the same id.
+
+        """
+        path = '/ermrest/alias'
+        r = deriva_server.post(path, json=cls._digest_alias_args(id, owner, alias_target))
+        r.raise_for_status()
+        return cls.connect(deriva_server, r.json()['id'])
+
+    def __init__(self, scheme, server, alias_id, credentials=None, caching=True, session_config=None):
+        """Create ERMrest alias binding.
+
+        :param scheme: 'http' or 'https'
+        :param server: server FQDN string
+        :param alias_id: e.g. '1'
+        :param credentials: credential secrets, e.g. cookie
+        :param caching: whether to retain a GET response cache
+
+        """
+        super(ErmrestAlias, self).__init__(scheme, server, credentials, caching, session_config)
+        self._server_uri = "%s/ermrest/alias/%s" % (
+            self._server_uri,
+            alias_id
+        )
+        self._scheme, self._server, self._alias_id, self._credentials, self._caching, self._session_config = \
+            scheme, server, alias_id, credentials, caching, session_config
+
+    @property
+    def alias_id(self):
+        return self._alias_id
+
+    def check_path(self, path):
+        if path != '':
+            raise ValueError('ErmrestAlias requires "" relative path')
+
+    def retrieve(self):
+        """Retrieve current alias binding state as a dict.
+
+        The returned dictionary is suitable for local revision and
+        being passed back into self.update:
+
+           state = self.retrieve()
+           state.update({ "owner": ..., "alias_target": ...)
+           self.update(**state)
+
+        """
+        return self.get('').json()
+
+    def update(self, owner=nochange, alias_target=nochange, id=None):
+        """Update alias binding state in server, returning the response message dict.
+
+        :param owner: Revised owner ACL for binding or nochange (default None)
+        :param alias_target: Revised target for binding or nochange (default None)
+        :param id: Current self.alias_id or None (default None)
+
+        The optional id parameter must be None or self.alias_id and
+        does not affect state changes to the server. It is only
+        specified in order to allow an idiom like:
+
+           state = self.retrieve()
+           state.update(...)
+           self.update(**state)
+
+        where the original "id" field of self.retrieve() is harmlessly
+        passed through as a keyword.
+
+        """
+        rep = self._digest_alias_args(id, owner, alias_target)
+        if id is not None and id != self.alias_id:
+            raise ValueError('parameter id must be None or %r, not %r' % (self.alias_id, id))
+        return self.put('', json=rep).json()
+
+    def delete_ermrest_alias(self, really=False):
+        """Perform DELETE request, destroying alias on server.
+
+        :param really: delete when True, abort when False (default)
+
+        """
+        if really is True:
+            return DerivaBinding.delete(self, '')
+        else:
+            raise ValueError('Alias deletion refused when really is %s.' % really)
+
