@@ -73,7 +73,11 @@ class HatracStore(DerivaBinding):
         else:
             return False
 
-    def get_obj(self, path, headers=DEFAULT_HEADERS, destfilename=None, callback=None):
+    def get_obj(self, path,
+                headers=DEFAULT_HEADERS,
+                destfilename=None,
+                callback=None,
+                chunk_size=DEFAULT_CHUNK_SIZE):
         """Retrieve resource optionally streamed to destination file.
 
            If destfilename is provided, download content to file with
@@ -107,13 +111,16 @@ class HatracStore(DerivaBinding):
 
             if destfilename is not None:
                 total = 0
+                current_chunk = 0
                 start = datetime.datetime.now()
                 logging.debug("Transferring file %s to %s" % (self._server_uri + path, destfilename))
-                for buf in r.iter_content(chunk_size=DEFAULT_CHUNK_SIZE):
+                for buf in r.iter_content(chunk_size=chunk_size):
                     destfile.write(buf)
                     total += len(buf)
+                    current_chunk += 1
                     if callback:
-                        if not callback(progress="Downloading: %.2f MB transferred" % (total / Megabyte)):
+                        if not callback(progress="Downloading: %.2f MB transferred" % (total / Megabyte),
+                                        total_bytes=total, current_chunk=current_chunk):
                             destfile.close()
                             r.close()
                             os.remove(destfilename)
@@ -121,7 +128,6 @@ class HatracStore(DerivaBinding):
                 elapsed = datetime.datetime.now() - start
                 summary = get_transfer_summary(total, elapsed)
                 destfile.flush()
-                os.fsync(destfile.fileno())
                 logging.info("File [%s] transfer successful. %s" % (destfilename, summary))
                 if callback:
                     callback(summary=summary, file_path=destfilename)
@@ -140,7 +146,7 @@ class HatracStore(DerivaBinding):
                     rmd5 = r.headers.get('Content-MD5', r.headers.get('content-md5', None))
                     if fmd5 != rmd5:
                         raise HatracHashMismatch('Content-MD5 %s != computed MD5 %s' % (rmd5, fmd5))
-
+                r.close()
             return r
         finally:
             if destfile is not None:
