@@ -17,14 +17,25 @@ from distutils import util as du_util
 IS_PY2 = (sys.version_info[0] == 2)
 IS_PY3 = (sys.version_info[0] == 3)
 
+
+def force_unicode_py2(s):
+    """Reliably return a Unicode string given a possible unicode or byte string"""
+    if isinstance(s, str):
+        return s.decode("utf-8")
+    else:
+        return unicode(s)
+
+
 if IS_PY3:
     from urllib.parse import quote as _urlquote, unquote as urlunquote
     from urllib.parse import urlparse, urlsplit, urlunsplit, urljoin
     from http.cookiejar import MozillaCookieJar
+    force_unicode = str
 else:
     from urllib import quote as _urlquote, unquote as urlunquote
     from urlparse import urlparse, urlsplit, urlunsplit, urljoin
     from cookielib import MozillaCookieJar
+    force_unicode = force_unicode_py2
 
 Kilobyte = 1024
 Megabyte = Kilobyte ** 2
@@ -252,9 +263,10 @@ def read_config(config_file=DEFAULT_CONFIG_FILE, create_default=False, default=D
     return json.loads(config, object_pairs_hook=OrderedDict)
 
 
-def lock_file(file_path, mode, exclusive=True):
-    return portalocker.Lock(file_path, mode, timeout=60,
-                            flags=portalocker.LOCK_EX if exclusive else portalocker.LOCK_SH)
+def lock_file(file_path, mode, exclusive=True, timeout=60):
+    return portalocker.Lock(file_path, mode=mode, timeout=timeout, fail_when_locked=True,
+                            flags=(portalocker.LOCK_EX | portalocker.LOCK_NB) if exclusive else
+                            (portalocker.LOCK_SH | portalocker.LOCK_NB))
 
 
 def write_credential(credential_file=DEFAULT_CREDENTIAL_FILE, credential=DEFAULT_CREDENTIAL):
@@ -417,6 +429,26 @@ def calculate_optimal_transfer_shape(size,
     return chosen_chunk_size, chunk_count, remainder
 
 
+def json_item_handler(input_file, callback):
+    with io.open(input_file, "r", encoding='utf-8') as infile:
+        line = infile.readline().lstrip()
+        infile.seek(0)
+        is_json_stream = False
+        if line.startswith('{') and line.endswith('}\n'):
+            input_json = infile
+            is_json_stream = True
+        else:
+            input_json = json.load(infile, object_pairs_hook=OrderedDict)
+        try:
+            for item in input_json:
+                if is_json_stream:
+                    item = json.loads(item, object_pairs_hook=OrderedDict)
+                if callback:
+                    callback(item)
+        finally:
+            infile.close()
+
+
 def topo_sorted(depmap):
     """Return list of items topologically sorted.
 
