@@ -6,7 +6,7 @@ INFO = "For more information see: https://github.com/informatics-isi-edu/deriva-
 
 
 def main():
-    cli = DerivaDownloadCLI(DESC, INFO)
+    cli = DerivaDownloadCLI(DESC, INFO, hostname_required=True, config_file_required=True)
     return cli.main()
 
 
